include_guard(GLOBAL)

list(APPEND CMAKE_MODULE_PATH "${CMAKE_CURRENT_LIST_DIR}")

enable_language(ASM)
set(CMAKE_EXECUTABLE_SUFFIX ".elf")

set(_EDX_ESCAPE "[")
set(_ESC_ERR "${_EDX_ESCAPE}31m")
set(_ESC_INFO "${_EDX_ESCAPE}32m")
set(_ESC_RST "${_EDX_ESCAPE}0m")
# for llvm
if ("${CMAKE_C_COMPILER}" MATCHES ".*/clang\\..*")
	set(CLANG 1)
elseif ("${CMAKE_C_COMPILER}" MATCHES ".*/.*-gcc\.exe")
	set(GCC 1)
endif()

if (GCC)
	execute_process(
		COMMAND ${CMAKE_C_COMPILER} -v
		OUTPUT_VARIABLE  C_COMPILER_V_OUTPUT
		ERROR_VARIABLE  C_COMPILER_V_OUTPUT
	)
	string(REGEX MATCH "Target:[ \\t]*([^ \t\n]*)" GCC_TARGET "${C_COMPILER_V_OUTPUT}")
	set(GCC_TARGET ${CMAKE_MATCH_1})
	string(REGEX MATCH "([^-]*)-([^-]*)-(.*)" GCC_TARGET_ARCH "${GCC_TARGET}")
	set(GCC_TARGET_ARCH ${CMAKE_MATCH_1})
	string(REPLACE "x86_64" "x64" GCC_TARGET_ARCH ${GCC_TARGET_ARCH})
	set(GCC_TARGET_OS ${CMAKE_MATCH_2})
	set(GCC_TARGET_ABI ${CMAKE_MATCH_3})

	string(REGEX MATCH "gcc version[ \\t]*([0-9\\.]+)" GCC_VERSION "${C_COMPILER_V_OUTPUT}")
	set(GCC_VERSION ${CMAKE_MATCH_1})
	if ("${GCC_VERSION}" VERSION_GREATER_EQUAL "11.0.0")
		SET(EMBEDED_GCC_EXTRA_LD_FLAGS
			-Wl,--no-warn-rwx-segments
		)
	endif()
endif()


set(_WARNING_PREFIX "       ${_ESC_ERR}Warning${_ESC_RST}")
if ("$ENV{SDK_TYPE}" STREQUAL "STM32Cube")
	include(${CMAKE_CURRENT_LIST_DIR}/stm32cube.cmake)
elseif("$ENV{SDK_TYPE}" STREQUAL "AirMCU")
	include(${CMAKE_CURRENT_LIST_DIR}/air32.cmake)
elseif("$ENV{SDK_TYPE}" STREQUAL "XHSC")
	include(${CMAKE_CURRENT_LIST_DIR}/xhsc.cmake)
elseif("$ENV{SDK_TYPE}" STREQUAL "GD32")
	include(${CMAKE_CURRENT_LIST_DIR}/gd32.cmake)
endif()
